/*
 * tut6.c -- SDL-Tutorial, 6. Kapitel
 * NO Copyright (c) by Nicolai 'Prefect' Haehnle
 *
 * Diese Software ist Public Domain.
 */

#include <stdlib.h>
#include <stdio.h>
#include <time.h>

#include <SDL.h>

SDL_Surface *g_pSurfScreen; /* Der Bildschirminhalt */
SDL_Surface *g_pSurfGalaxies; /* galaxien.bmp */
SDL_Surface *g_pSurfShip; /* ship.bmp */

Uint32 g_Black; /* Farbwert schwarz */

struct { /* Zustand des Raumschiffs in der Bildschirmmitte */
	int rotation;
	Uint32 nextturn;
} ship;

/* Wieviele Millisekunden liegen zwischen den einzelnen Drehschritten? */
#define TURN_DELAY		75

/* Wie viele Galaxien sind auf dem Bildschirm? */
#define NUM_GALAXIES		100

struct galaxy {
	int x, y;	/* Koordinaten der Galaxie */
	int depth;	/* "Tiefe" der Galaxie */
	int type; 	/* Welches Bild wird fr die Galaxien verwendet? */
};

static int g_DepthCount[] = {
	0,
	0,
	0,
	0,
	20,	/* volle Geschwindigkeit */
	34,
	45,
	55,
	64,	/* 1/2 */
	72,
	79,
	85,
	90,	/* 1/3 */
	94,
	97,
	99,
	100	/* 1/4 */
};

struct galaxy g_Galaxies[NUM_GALAXIES];

/* Erstelle ein Array aus zufllig gewhlten Galaxienpositionen */
void RandBackground()
{
	int i;
	int num_types; /* Anzahl Galaxientypen im Bitmap "galaxien.bmp" */
	int depth;

	num_types = g_pSurfGalaxies->w / 16;

/* Flle das Galaxien-Array mit zufllig gewhlten Werten */
	depth = 0;
	for(i = 0; i < NUM_GALAXIES; i++) {
		g_Galaxies[i].x = rand() % 1024;
		g_Galaxies[i].y = rand() % 512;
		g_Galaxies[i].type = rand() % num_types;

		while(i >= g_DepthCount[depth])
			depth++;
		g_Galaxies[i].depth = depth;
	}
}

/* Zeichne einen Sternenhintergrund auf den Bildschirm */
void DrawBackground(int scrollx, int scrolly)
{
	int i;

	for(i = NUM_GALAXIES-1; i >= 0; i--) {
		SDL_Rect src;
		SDL_Rect dest;

	/* Berechne die Position des Galaxientypbilds im Quellbitmap */
		src.x = g_Galaxies[i].type * 16;
		src.y = 0;
		src.w = 16;
		src.h = 16;

	/* Die Zielkoordinaten werden eingefllt */
		dest.x = ((unsigned int)(g_Galaxies[i].x - (4*scrollx / g_Galaxies[i].depth)) % 1024) - 16;
		dest.y = ((unsigned int)(g_Galaxies[i].y - (4*scrolly / g_Galaxies[i].depth)) % 512) - 16;

		SDL_BlitSurface(g_pSurfGalaxies, &src, g_pSurfScreen, &dest);
	}
}

/* Zeichne ein normales Sprite mit 36 Drehschritten */
void DrawSprite(SDL_Surface *pSurf, int dx, int dy, int frame, int rotation)
{
	SDL_Rect src;
	SDL_Rect dest;

	src.x = (rotation % 9) * 49;
	src.y = ((rotation / 9) + (4 * frame)) * 49;
	src.w = 48;
	src.h = 48;

	dest.x = dx - 24;
	dest.y = dy - 24;

	SDL_BlitSurface(pSurf, &src, g_pSurfScreen, &dest);
}

/* Eine einfache Wrapperfunktion zum Laden von Bitmaps */
SDL_Surface *LoadBMP(const char *szFile)
{
	SDL_Surface *orig, *convert;

/* Lade das Bitmap */
	orig = SDL_LoadBMP(szFile);
	if (!orig) {
		fprintf(stderr, "%s konnte nicht geladen werden: %s\n", szFile, SDL_GetError());
		exit(1);
	}

/* Konvertiere ins Bildschirmformat fr bessere Performance */
	convert = SDL_ConvertSurface(orig, g_pSurfScreen->format, g_pSurfScreen->flags);
	SDL_FreeSurface(orig); /* alte Surface wird freigegeben */
	if (!convert) {
		fprintf(stderr, "%s konnte nicht ins Bildschirmformat konvertiert werden: %s\n",
			szFile, SDL_GetError());
		exit(1);
	}

	return convert;
}

#ifdef _WIN32
#undef main
#endif
int main()
{
	int running;
	int scrollx, scrolly;
	Uint32 lastframe, curframe, frametime;

/* Initialisiere den Zufallszahlengenerator */
	srand(time(0));

/* SDL initialisieren */
	if (SDL_Init(SDL_INIT_VIDEO) < 0) {
		fprintf(stderr, "SDL konnte nicht initialisiert werden: %s\n",
			SDL_GetError());
		exit(1);
	}
	atexit(SDL_Quit);

/* Bildschirmmodus setzen */
	g_pSurfScreen = SDL_SetVideoMode(640, 480, 0, SDL_DOUBLEBUF);
	if (!g_pSurfScreen) {
		fprintf(stderr, "Konnte Bildschirmmodus nicht setzen: %s\n",
			SDL_GetError());
		exit(1);
	}

/* Maus einsperren */
	SDL_WM_GrabInput(SDL_GRAB_ON);

/* Farbwerte bestimmen */
	g_Black = SDL_MapRGB(g_pSurfScreen->format, 0, 0, 0);

/* galaxien.bmp laden */
	g_pSurfGalaxies = LoadBMP("galaxien.bmp");
	SDL_SetColorKey(g_pSurfGalaxies, SDL_SRCCOLORKEY, g_Black);

/* ship.bmp laden */
	g_pSurfShip = LoadBMP("ship.bmp");
	SDL_SetColorKey(g_pSurfShip, SDL_SRCCOLORKEY, g_Black);

/* Galaxienpositionen vorprogrammieren */
	RandBackground();

/* Der Timer mu initialisiert werden */
	curframe = SDL_GetTicks();

/* Programmschleife */
	running = 1;
	scrollx = scrolly = 0;
	ship.rotation = 0;
	ship.nextturn = curframe + TURN_DELAY;
	while(running) {
		SDL_Event event;
		Uint8 *keystate;
		int mousex, mousey;

		while(SDL_PollEvent(&event)) {
			switch(event.type) {
			case SDL_KEYDOWN:
				switch(event.key.keysym.sym) {
				/* Beim Druck von ESC beendet sich das Programm */
				case SDLK_ESCAPE:
					running = 0;
					break;

				/* Alle anderen Tasten werden ignoriert */
				default:
					break;
				}
				break;

			/* Wenn die Maus bewegt wird whrend der rechte Maustaste
			   gedrckt ist scrollt der Bildschirm mit */
			case SDL_MOUSEMOTION:
				if (event.motion.state & SDL_BUTTON(SDL_BUTTON_RIGHT)) {
					scrollx -= event.motion.xrel;
					scrolly -= event.motion.yrel;
				}
				break;

			/* Bei einem Druck auf die linke Maustaste zentriert der
			   Bildschirm auf der momentanen Mausposition */
			case SDL_MOUSEBUTTONDOWN:
				if (event.button.button == SDL_BUTTON_LEFT) {
					scrollx += event.button.x - 320;
					scrolly += event.button.y - 240;
				}
				break;

			case SDL_QUIT:
				running = 0;
				break;
			}
		}

	/* Frametime wird berechnet */
		lastframe = curframe;
		curframe = SDL_GetTicks();
		frametime = curframe - lastframe;

	/* Momentanen Tastaturzustand und Mausposition holen und entsprechend
	   scrollen */
		keystate = SDL_GetKeyState(0);
		SDL_GetMouseState(&mousex, &mousey);

		if (keystate[SDLK_LEFT] || mousex < 16)
			scrollx -= (400 * frametime)/1000;
		if (keystate[SDLK_RIGHT] || mousex > 640-16)
			scrollx += (400 * frametime)/1000;
		if (keystate[SDLK_UP] || mousey < 16)
			scrolly -= (400 * frametime)/1000;
		if (keystate[SDLK_DOWN] || mousey > 480-16)
			scrolly += (400 * frametime)/1000;

    /* Zustand des Raumschiffes updaten */
		while(ship.nextturn < curframe) {
			if (++ship.rotation >= 36)
				ship.rotation = 0;
			ship.nextturn += TURN_DELAY;
		}

	/* Bildschirmhintergrund lschen */
		SDL_FillRect(g_pSurfScreen, 0, g_Black);

	/* Sternenhintergrund darstellen */
		DrawBackground(scrollx, scrolly);

	/* Raumschiff darstellen */
		DrawSprite(g_pSurfShip, 320, 240, keystate[SDLK_SPACE] ? 1 : 0, ship.rotation);

	/* Die nderung mssen wirksam gemacht werden */
		SDL_Flip(g_pSurfScreen);
	}

/* Unsere Bitmaps mssen freigegeben werden */
	SDL_FreeSurface(g_pSurfShip);
	SDL_FreeSurface(g_pSurfGalaxies);

	return 0;
}
